"""Wordly

"""
from wordly.words import Word

__app_name__ = "Wordly"
__app_description__ = "A simple python client that requests dictionary definitions from a server that implements Dictionary Server Protocol"
__epilog__ = "Stay in the know."
__version__ = "0.0.1"

__all__ = ["Word"]



import argparse
import asyncio
from collections.abc import Sequence
from . import __app_name__, __app_description__, __version__, __epilog__
from wordly.words import Word


async def print_definition(word: str, hostname: str, port: int):
    w = Word(word, hostname=hostname, port=port)
    definition = await w.adefinition
    print(definition)

async def main(argv: Sequence[str]|None = None):
    parser = argparse.ArgumentParser(
        prog=__app_name__,
        description=__app_description__,
        epilog=__epilog__,
    )
    opt = parser.add_argument

    opt("words", nargs="+", help="One or more words to define.")
    opt("-v", "--version", version=__version__, action="version")
    opt("-p", "--port", type=int, default=2628, help="Specify the port number. Default: 2628")
    opt("-H", "--hostname", type=str, default="dict.org", help="Specify the server. Default: dict.org")

    args = vars(parser.parse_args(argv))

    tasks = [
        print_definition(word, args["hostname"], args["port"])
        for word in args["words"]
    ]

    await asyncio.gather(*tasks)



from collections import defaultdict
from wordly.status_codes import Status


class DictParser:
    def __init__(self, delimiter: bytes = b"\r\n"):
        self.line = b""
        self.mapping = defaultdict(str)
        self.DELIMITER = delimiter

    def _process_line(self, ending: str = ""):
        code = self.line[:3]

        if (status := Status.by_value(code)):
            self.mapping[status.name] += self.line[4:].decode() + ending
            self.part = status.name
        else:
            self.mapping[self.part] += self.line.decode() + ending

    def feed(self, stream: bytes):
        split = stream.split(self.DELIMITER, 1)
        while len(split) > 1:
            old, new = split
            self.line += old
            self._process_line("\n")
            self.line = b""
            split = new.split(self.DELIMITER, 1)

        if (line := split[0]):
            self.line += line
            self._process_line()
            self.line = b""



import asyncio
from collections import UserString
from wordly.client import DictClient
from wordly.status_codes import Status


class Word(UserString):
    def __init__(self, seq, hostname: str = "dict.org", port: int = 2628, client = None):
        super().__init__(seq)

        self.client = client or DictClient(hostname=hostname, port=port)
        self._cache = {}

    @property
    async def adefinition(self):
        if not self._cache:
            data = await self.client.define(self.data)
            self._cache.update(data.mapping)
        return self._cache.get(Status.DEFINITION.name)

    @property
    def definition(self):
        if not self._cache:
            loop = asyncio.get_event_loop()
            data = loop.run_until_complete(self.client.define(self.data))
            self._cache.update(data.mapping)
        return self._cache.get(Status.DEFINITION.name)



import unittest
from wordly.parser import DictParser
from wordly.status_codes import Status
from tests.constants import HELP_OUTPUT, PROGRAMMING_DEFINITION


class TestParser(unittest.TestCase):
    def setUp(self) -> None:
        self.line_reader = DictParser()

    def test_help(self):
        self.line_reader.feed(HELP_OUTPUT)

        self.assertEqual(self.line_reader.mapping[Status.INITIAL_CONNECTION.name], "banner info from example.org\n")
        self.assertEqual(self.line_reader.mapping[Status.COMMAND_COMPLETE.name], "ok\n")
        self.assertEqual(self.line_reader.mapping[Status.CLOSING_CONNECTION.name], "bye [d/m/c = 0/0/0; 0.000r 0.000u 0.000s]")

    def test_define(self):
        self.line_reader.feed(PROGRAMMING_DEFINITION)

        self.assertEqual(self.line_reader.mapping[Status.INITIAL_CONNECTION.name], "banner information contained here\n")
        self.assertEqual(self.line_reader.mapping[Status.CLOSING_CONNECTION.name], "bye [d/m/c = 0/0/0; 0.000r 0.000u 0.000s]"
)

